"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DOMRenderer = void 0;
var tslib_1 = require("tslib");
var web_utility_1 = require("web-utility");
var VDOM_1 = require("./VDOM");
var DOMRenderer = /** @class */ (function () {
    function DOMRenderer() {
        var _this = this;
        this.eventPattern = /^on[A-Z]/;
        this.ariaPattern = /^aira[A-Z]/;
        this.treeCache = new WeakMap();
        this.keyOf = function (_a, index) {
            var key = _a.key, text = _a.text, props = _a.props, selector = _a.selector;
            return (key === null || key === void 0 ? void 0 : key.toString()) || (props === null || props === void 0 ? void 0 : props.id) || (text || selector || '') + index;
        };
        this.vNodeOf = function (list, key) {
            return list.find(function (vNode, index) { return "".concat(_this.keyOf(vNode, index)) === String(key); });
        };
        this.propsKeyOf = function (key) {
            return key.startsWith('aria-')
                ? (0, web_utility_1.toCamelCase)(key)
                : _this.eventPattern.test(key)
                    ? key.toLowerCase()
                    : key;
        };
    }
    DOMRenderer.prototype.updateProps = function (node, oldProps, newProps, onDelete, onAdd) {
        var e_1, _a, e_2, _b;
        if (oldProps === void 0) { oldProps = {}; }
        if (newProps === void 0) { newProps = {}; }
        var group = (0, web_utility_1.diffKeys)(Object.keys(oldProps), Object.keys(newProps)).group;
        try {
            for (var _c = tslib_1.__values(group[web_utility_1.DiffStatus.Old] || []), _d = _c.next(); !_d.done; _d = _c.next()) {
                var _e = tslib_1.__read(_d.value, 1), key = _e[0];
                onDelete === null || onDelete === void 0 ? void 0 : onDelete(node, key);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_1) throw e_1.error; }
        }
        try {
            for (var _f = tslib_1.__values(tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read((group[web_utility_1.DiffStatus.Same] || [])), false), tslib_1.__read((group[web_utility_1.DiffStatus.New] || [])), false)), _g = _f.next(); !_g.done; _g = _f.next()) {
                var _h = tslib_1.__read(_g.value, 1), key = _h[0];
                if (oldProps[key] !== newProps[key])
                    if (onAdd instanceof Function)
                        onAdd(node, key, newProps[key]);
                    else
                        Reflect.set(node, key, newProps[key]);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_g && !_g.done && (_b = _f.return)) _b.call(_f);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    DOMRenderer.prototype.createNode = function (vNode, reusedVNodes) {
        var _a, _b;
        if (vNode.text)
            return (vNode.node = document.createTextNode(vNode.text));
        var reusedVNode = vNode.selector && ((_a = reusedVNodes === null || reusedVNodes === void 0 ? void 0 : reusedVNodes[vNode.selector]) === null || _a === void 0 ? void 0 : _a.shift());
        vNode.node = vNode.tagName
            ? (reusedVNode === null || reusedVNode === void 0 ? void 0 : reusedVNode.node) ||
                document.createElement(vNode.tagName, { is: vNode.is })
            : document.createDocumentFragment();
        var node = this.patch(reusedVNode || { tagName: vNode.tagName, node: vNode.node }, vNode).node;
        if (node)
            (_b = vNode.ref) === null || _b === void 0 ? void 0 : _b.call(vNode, node);
        return node;
    };
    DOMRenderer.prototype.deleteNode = function (_a) {
        var ref = _a.ref, node = _a.node, children = _a.children;
        if (node instanceof DocumentFragment)
            children === null || children === void 0 ? void 0 : children.forEach(this.deleteNode);
        else if (node) {
            node.remove();
            ref === null || ref === void 0 ? void 0 : ref();
        }
    };
    DOMRenderer.prototype.commitChildren = function (root, newNodes) {
        var e_3, _a;
        try {
            for (var _b = tslib_1.__values(tslib_1.__spreadArray([], tslib_1.__read(root.childNodes), false)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var oldNode = _c.value;
                var index = newNodes.indexOf(oldNode);
                if (index < 0)
                    continue;
                else if (index === 0) {
                    newNodes.shift();
                    continue;
                }
                var beforeNodes = newNodes.slice(0, index);
                if (!beforeNodes[0])
                    continue;
                oldNode.before.apply(oldNode, tslib_1.__spreadArray([], tslib_1.__read(beforeNodes), false));
                newNodes = newNodes.slice(index + 1);
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
        if (newNodes[0])
            root.append.apply(root, tslib_1.__spreadArray([], tslib_1.__read(newNodes), false));
    };
    DOMRenderer.prototype.updateChildren = function (node, oldList, newList) {
        var e_4, _a;
        var _this = this;
        var _b = (0, web_utility_1.diffKeys)(oldList.map(this.keyOf), newList.map(this.keyOf)), map = _b.map, group = _b.group;
        var deletingGroup = group[web_utility_1.DiffStatus.Old] &&
            (0, web_utility_1.groupBy)(group[web_utility_1.DiffStatus.Old].map(function (_a) {
                var _b = tslib_1.__read(_a, 1), key = _b[0];
                return _this.vNodeOf(oldList, key);
            }), function (_a) {
                var selector = _a.selector;
                return selector + '';
            });
        var newNodes = newList.map(function (vNode, index) {
            var key = _this.keyOf(vNode, index);
            if (map[key] !== web_utility_1.DiffStatus.Same)
                return _this.createNode(vNode, deletingGroup);
            var oldVNode = _this.vNodeOf(oldList, key);
            return vNode.text != null
                ? (vNode.node = oldVNode.node)
                : _this.patch(oldVNode, vNode).node;
        });
        for (var selector in deletingGroup)
            try {
                for (var _c = (e_4 = void 0, tslib_1.__values(deletingGroup[selector])), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var vNode = _d.value;
                    this.deleteNode(vNode);
                }
            }
            catch (e_4_1) { e_4 = { error: e_4_1 }; }
            finally {
                try {
                    if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
                }
                finally { if (e_4) throw e_4.error; }
            }
        this.commitChildren(node, newNodes);
    };
    DOMRenderer.prototype.patch = function (oldVNode, newVNode) {
        var _this = this;
        var tagName = oldVNode.tagName;
        var isXML = (0, web_utility_1.templateOf)(tagName) && (0, web_utility_1.elementTypeOf)(tagName) === 'xml';
        this.updateProps(oldVNode.node, oldVNode.props, newVNode.props, function (node, key) {
            return _this.eventPattern.test(key)
                ? (node[key.toLowerCase()] = null)
                : node.removeAttribute(_this.ariaPattern.test(key)
                    ? (0, web_utility_1.toHyphenCase)(key)
                    : VDOM_1.VNode.propsMap[key] || key);
        }, function (node, key, value) {
            // @ts-ignore
            if (isXML || key.includes('-') || (0, web_utility_1.isDOMReadOnly)(tagName, key))
                node.setAttribute(key, value);
            else
                node[_this.propsKeyOf(key)] = value;
        });
        this.updateProps(oldVNode.node.style, oldVNode.style, newVNode.style, function (node, key) { return node.removeProperty(key); });
        this.updateChildren(oldVNode.node, oldVNode.children || [], newVNode.children || []);
        newVNode.node = oldVNode.node;
        return newVNode;
    };
    DOMRenderer.prototype.render = function (vNode, node) {
        if (node === void 0) { node = document.body; }
        var root = this.treeCache.get(node) || VDOM_1.VNode.fromDOM(node);
        root = this.patch(root, tslib_1.__assign(tslib_1.__assign({}, root), { children: [vNode] }));
        this.treeCache.set(node, root);
        return root;
    };
    DOMRenderer.prototype.renderToStaticMarkup = function (tree) {
        var body = document.implementation.createHTMLDocument().body;
        this.render(tree, body);
        return body.innerHTML;
    };
    return DOMRenderer;
}());
exports.DOMRenderer = DOMRenderer;
